// UMD Econ-626
// PS5
// Wild Cluster Bootstrap

clear all

// set initial seed
set seed 7654
// number of iterations
local itermax=1000
// number of clusters
local clusternum=20
// number of clusters to be treated
local treatclusters=4
// number of observations per cluster
local obspercluster=25
// true effect size (zero for null)
local trueffect=0

// number of boostrap iterations
local bootstrapmax=100


// timing
timer clear 1
timer on 1


// set number of clusters
set obs `clusternum'
// generate cluster identifier for convenience
gen g=_n
// generate treatment variable, varying at cluster level
gen t=cond(g<=`treatclusters',1,0)
// for each iteration:
forvalues iter=1/`itermax' {
 // report every 10 
 if(mod(`iter',10)==0) {
   di "Iteration `iter' of `itermax'"
 }
 // generate a random (standard normal) shock term at the cluster level
 sort g
 gen nu=invnorm(uniform())
 // expand the dataset so that there are multiple observations per cluster
 qui expand `obspercluster'
 // generate identifier within clusterfor convenience
 bys g: gen within_n=_n
 // generate a random (standard normal) shock term at the individual level
 gen eta=invnorm(uniform())
 // generate outcome = sum of the two shocks. (no effect of treatment under the null)
 gen y=eta+nu
 qui replace y=y+`trueffect'*t // or with an effect
 // do the analysis without clustering (robust); store p-value in local to be accessed later
 qui reg y t, r
 matrix rt=r(table)
 local p1_`iter'=rt[4,1]
 // do the analysis clustering; store p-value in local to be accessed later
 qui reg y t, vce(cluster g)
 local w_orig=_b[t]/_se[t] // may need ORIGINAL t statistic for bootstrap.
 matrix rt=r(table)
 local p2_`iter'=rt[4,1]
 
 
 
 // BEGIN WCB CODE
 
 // do the analysis WCB; store p-value in local to be accessed later
 
 // 1. estimate but forcing null.
 qui reg y
 predict U, resid
 predict yhatimposed, xb
 
 
 // create a local variable to keep track of how many times the absolute value of a bootstrap t statistic exceeds the absolute value of the original T statistic.
 local thispsum=0

 // for each bootstrap iteration, following Cameron, Gelbach, Miller 2008 RESTAT 90(3): 414-427 Appendix B procedure for wild cluster bootstrap:
 // YOUR CODE (POSSIBLE MULTIPLE LINES) BETWEEN LINES BELOW:
 forvalues biter=1/`bootstrapmax' {
 
   // generate random weights of either 1 or -1 (each with probability 0.5) for each cluster
   // * YOUR CODE HERE 

   // multiply that weight by the residual, U, to create Ustar.
   // * YOUR CODE HERE 
   
   // add Ustar to yhatimposed to generate ystar.
   // * YOUR CODE HERE 
   
   // regress ystar on t, clustering standard errors (at the group level, "g")
   // * YOUR CODE HERE 
   
   // this bootstrap iteration's t-statistic is _b[t]/_se[t] from the regression above.
   // * YOUR CODE HERE 
   
   // increment the "thispsum" counter only if the absolute value of the current bootstrap iteration's t-statistic is larger than the absolute value of the original T statistic.
   // * YOUR CODE HERE 
 }
 // end bootstrap loop.
 // the bootstrap p-value do the analysis without clustering (robust); store p-value in local to be accessed later
 // * YOUR CODE HERE 

 // CHECK THE WILD CLUSTER BOOTSTRAP P-VALUE YOU GOT.
 // use the following command (you will have to install it):
 //  boottest t, boot(wild) cluster(g) nogr noci
 // save this p-value as well.
 // * YOUR CODE HERE 

 // drop all variables created since the line marked "BEGIN WCB CODE"
 cap drop U
 cap drop yhatimposed
 // * YOUR CODE HERE 
 
 // END WCB CODE
 
 
 // drop all but one obs per group
 cap drop if within_n!=1
 // drop all the variables generated after t and g
 cap drop within_n
 cap drop eta
 cap drop y
 cap drop nu
}

// timing
timer off 1
timer list 1


// now make a dataset of the p-values from the locals
clear
set obs `itermax'
gen p1=.
gen p2=.
// * YOUR CODE HERE, OR UNCOMMENT THESE LINES AS NEEDED
//gen p3=. // wcb
//gen p4=. // wcb
forvalues iter=1/`itermax' {
 qui replace p1=`p1_`iter'' in `iter'
 qui replace p2=`p2_`iter'' in `iter'
// * YOUR CODE HERE, OR UNCOMMENT THESE LINES AS NEEDED
// qui replace p3=`p3_`iter'' in `iter' // wcb
// qui replace p4=`p4_`iter'' in `iter' // wcb
}

count if p1<0.05
count if p2<0.05
// * YOUR CODE HERE, OR UNCOMMENT THESE LINES AS NEEDED
//count if p3<0.05 // wcb
//count if p4<0.05 // wcb
hist p2


